/****************************************************************************
**  DerBrowserTimer - Timer/Clock/Alarm add-on for Mozilla FireFox
**  Copyright (c) 2004  Charles Melhorn
**
**  This file is part of DerBrowserTimer.
**
**  DerBrowserTimer is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  DerBrowserTimer is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 


const gDBTconstants =
{
  AS_OFF:          0,  // alarm states
  AS_SET:          1,
  AS_ALARMING:     2,

  AT_TIMER:        0,  // alarm types
  AT_ALARM:        1,
  AT_PERIODIC:     2,

  TP_QUARTERHR:    15, // target periods
  TP_HALFHR:       30,
  TP_HOURLY:       60,

  CF_12HR:         0,  // clock formats
  CF_24HR:         1,

  CI_HOUR:         0,  // cuckoo interval
  CI_HALFHOUR:     1,

  DT_TIME:         0,  // time display type
  DT_DATE:         1,
  DT_REMAIN:       2
}


var gDBTClockController = 
{
  timeDisplayType:   gDBTconstants.DT_TIME,

  bAlarmSet:         false,
  alarmState:        gDBTconstants.AS_OFF,
  alarm_type:        gDBTconstants.AT_TIMER,
  targetDate:        null,
  origTargetDate:    null,
  countdown_time:    0,
  target_period:     gDBTconstants.TP_HOURLY,
  period_offset:     0,
  snooze_increment:  1,
  auto_reset:        false,
  reminder_msg:      "",

  bCuckooOn:         false,

  sleep_timer:       false
}

var gCuckoo_played = false;
var gAlarmcycle_cnt = 0;

var gClock_TimerID = setInterval( "checkTime()", 1000 );
var gAlarm_TimerID = null;


function dateToClock(theDate)
{
   var formatpref = gDBTUserPreferenceInfo.getPrefValue("clock_format");

   var hour = theDate.getHours();
   var minute = theDate.getMinutes();
   var temp = "";

   if ( formatpref == gDBTconstants.CF_12HR)
    {
      temp += ((hour > 12) ? hour - 12 : hour);

      if (hour == 0)
         temp = "12";
      temp += ((minute < 10) ? ":0" : ":") + minute;
      temp += (hour >= 12) ? " PM" : " AM";
    }
   else
      temp += ((hour < 10) ? "0" : "") + hour + ((minute < 10) ? ":0" : ":") + minute;


   return temp;
}


function shortDate(currentDate)
{
  var temp = "" + (currentDate.getMonth() + 1) + "/"
             + currentDate.getDate() + "/" + currentDate.getFullYear();

  return temp;
}


function calculateTimeRemaining(currenttime)
{
  var time_remaining_secs = Math.floor((gDBTClockController.targetDate.getTime() - currenttime.getTime()) / 1000);
  var total_remaining_mins = Math.ceil(time_remaining_secs / 60);
  var time_remaining_hrs = Math.floor(total_remaining_mins / 60);
  var time_remaining_mins = total_remaining_mins % 60;

  var temp = "" + ((time_remaining_hrs < 10) ? "0" : "") + time_remaining_hrs;
  temp += ((time_remaining_mins < 10) ? ":0" : ":") + time_remaining_mins;

  return temp;
}


function checkTime()
{
  //alert("Checking Time"); 
  var currenttime = new Date();
 
  switch (gDBTClockController.timeDisplayType)
  {
    case gDBTconstants.DT_TIME:
      document.getElementById("dbt-menubarclock-lbl").value = dateToClock(currenttime);
      break;
    case gDBTconstants.DT_DATE:
      document.getElementById("dbt-menubarclock-lbl").value = shortDate(currenttime);
      break;
    case gDBTconstants.DT_REMAIN:
      if (gDBTClockController.alarmState == gDBTconstants.AS_SET)
        document.getElementById("dbt-menubarclock-lbl").value = calculateTimeRemaining(currenttime);
      else
        document.getElementById("dbt-menubarclock-lbl").value = "00:00";
      break;
    default:
      break;
  }

  if (gDBTClockController.alarmState == gDBTconstants.AS_SET)
   {
     var c_time_in_secs = Math.floor(currenttime.getTime() / 1000);
     var t_time_in_secs = Math.floor(gDBTClockController.targetDate.getTime() / 1000);

     if (c_time_in_secs >= t_time_in_secs)
      {
        gDBTClockController.alarmState = gDBTconstants.AS_ALARMING;
        document.getElementById("dbt-alarmicon-img").setAttribute("alarming", "true");
        document.getElementById("dbt-alarmicon-img").setAttribute("tooltiptext", "click to snooze");
        playAlarmSound();
        gAlarmcycle_cnt = 1;
        gAlarm_TimerID = setInterval( "signalAlarm()", 5000 );

        if (gDBTUserPreferenceInfo.getPrefValue("alarm_popwindow") == true)
         {
          window.openDialog("chrome://derbrowsertimer/content/dialogAlarmNotify.xul",
                            "dbtAlarmNotify","chrome,centerscreen,modal", gDBTClockController.reminder_msg);
         }
      }

   }

  // alarm takes precedence over cuckoo function, so don't bother
  // to check whether to cuckoo if alarm is firing

  if ( (gDBTClockController.bCuckooOn) &&
       (gDBTClockController.alarmState != gDBTconstants.AS_ALARMING) )
    {
      var raw_mins = currenttime.getMinutes();
      var mins = raw_mins;

      if (gDBTUserPreferenceInfo.getPrefValue("cuckoo_interval") == gDBTconstants.CI_HALFHOUR)
        mins = mins % 30;

      if ((mins == 0) && (gCuckoo_played == false))
       { 
         var hours = currenttime.getHours() % 12;
         hours = ((hours == 0) ? 12 : hours);
         var chimes = ((raw_mins == 30) ? 1 : hours);
         document.getElementById('dbt-cuckooicon-img').setAttribute("alarming", "true");
         SignalCuckoo(chimes);
         gCuckoo_played = true;
       }
      else if ((mins != 0) && (gCuckoo_played == true))
         gCuckoo_played = false;
    }

}


function playAlarmSound()
{
  var alarmsoundtype = gDBTUserPreferenceInfo.getPrefValue("alarm_sound_type");
  var alarmsoundfile = gDBTUserPreferenceInfo.getPrefValue("alarm_sound_file");

  if (alarmsoundtype == "standard")
   {
      playSoundURL("chrome://derbrowsertimer/content/" + alarmsoundfile);
   }
  else if (alarmsoundtype == "custom")
   {
      playSoundURL(alarmsoundfile);
   }
}


function playSoundURL(sURL_str)
{
  var sample = Components.classes["@mozilla.org/sound;1"].createInstance();
  sample = sample.QueryInterface(Components.interfaces.nsISound);
  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Components.interfaces.nsIIOService);
  var url = ioService.newURI(sURL_str, null, null);
  sample.play(url);

}


function ResetAlarm()
{
  if (gDBTClockController.alarm_type == gDBTconstants.AT_TIMER)
   {
     gDBTClockController.targetDate.setTime(gDBTClockController.targetDate.getTime() +
                                            gDBTClockController.countdown_time * 60 * 1000);
   }
  else if (gDBTClockController.alarm_type == gDBTconstants.AT_ALARM)
   {
     // targetDate might have been changed by snoozing, so
     // use original target date
     gDBTClockController.targetDate.setTime(gDBTClockController.origTargetDate.getTime() + 24*60*60*1000);
     gDBTClockController.origTargetDate.setTime(gDBTClockController.targetDate.getTime());
   }
  else  // gDBTClockController.alarm_type == gDBTconstants.AT_PERIODIC)
   {
     var targetdate = new Date();
     var current_minutes = targetdate.getMinutes();
     //alert("curr minutes: " + current_minutes);
     targetdate.setSeconds(0);
     targetdate.setMilliseconds(0);

     var mins_to_next_period = (gDBTClockController.target_period -
                               (current_minutes % gDBTClockController.target_period));

     var adjusted_for_offset_mins = (mins_to_next_period + gDBTClockController.period_offset)
                                     % gDBTClockController.target_period;

     if (adjusted_for_offset_mins == 0)
       adjusted_for_offset_mins = gDBTClockController.target_period;

     var target_adjust_ms = adjusted_for_offset_mins*60*1000;

     gDBTClockController.targetDate.setTime(targetdate.getTime() + target_adjust_ms);
   }

}


function signalAlarm()
{ 
  if (gAlarmcycle_cnt++ < gDBTUserPreferenceInfo.getPrefValue("alarm_timeout"))
   {
     playAlarmSound();
   }

  else // alarm has timed out
   {
     if (gDBTUserPreferenceInfo.getPrefValue("auto_reset") == true)
      {
        ResetAlarm();

        gDBTClockController.alarmState = gDBTconstants.AS_SET;
        document.getElementById("dbt-alarmicon-img").setAttribute("alarming", "false");
        alarmtime_str = "Alarm Time: " + dateToClock(gDBTClockController.targetDate);
        document.getElementById("dbt-alarmicon-img").setAttribute("tooltiptext", alarmtime_str);

        if (gAlarm_TimerID != null)
          clearInterval(gAlarm_TimerID);
      }
     else
      {
        dbtToggleAlarm("true");
      }
   }
}


/*****************************************************************
*  SignalCuckoo - called when cuckoo is first triggered (on hour *
*           and/or half hour depending on user preference), then *
*           calls itself recursively, pausing each time, until   *
*           the appropriate number of chimes have been sounded.  *
*           When no more chimes remain, it pauses to let the     *
*           last chime finish, then resets the cuckoo indicator  *
*           icon.                                                *
*                                                                *
*****************************************************************/

function SignalCuckoo(chimes_remaining)
{ 
  playCuckooSound(gDBTUserPreferenceInfo.getPrefValue("cuckoo_sound_file"));
  chimes_remaining--;

  if (chimes_remaining > 0)
   {
     setTimeout(SignalCuckoo, 750, chimes_remaining);
   }
  else
   {
     setTimeout("(document.getElementById('dbt-cuckooicon-img')).setAttribute('alarming', 'false');", 1000);
   }
}


function playCuckooSound(sfile_str)
{
  var sample = Components.classes["@mozilla.org/sound;1"].createInstance();
  sample = sample.QueryInterface(Components.interfaces.nsISound);
  const SND_NETWORK_STD_CID = "@mozilla.org/network/standard-url;1";
  const SND_I_URL           = "nsIURL";
  const SND_URL             = new Components.Constructor(SND_NETWORK_STD_CID, SND_I_URL);
  var url                   = new SND_URL();
  url.spec                  = "chrome://derbrowsertimer/content/" + sfile_str;
  sample.play(url);

}


function dbtEnableCuckoo(bEnable_str)
{
  if (bEnable_str == "true")
   {
     gDBTClockController.bCuckooOn = true;
     document.getElementById("dbt-cuckooicon-img").setAttribute("disabled", "false");
   }
  else
   {
     gDBTClockController.bCuckooOn = false;
     document.getElementById("dbt-cuckooicon-img").setAttribute("disabled", "true");
   }
}


function dbtToggleAlarm(bEnable_str)
{
  if (bEnable_str == "false")
   {
     var alarmSettings = 
     {
       bAlarmSet:         false,
       alarm_type:        gDBTconstants.AT_TIMER,
       targetDate:        null,
       origTargetDate:    null,
       countdown_time:    0,
       target_period:     gDBTconstants.TP_HOURLY,
       period_offset:     0,
       auto_reset:        false,
       snooze_increment:  0,
       reminder_msg:      ""
     }

     window.openDialog("chrome://derbrowsertimer/content/dialogAlarmSet.xul",
                       "dbtAlarmSet","chrome,modal,centerscreen", alarmSettings);

     if (alarmSettings.bAlarmSet)
      {
        gDBTClockController.alarmState = gDBTconstants.AS_SET;
        gDBTClockController.alarm_type = alarmSettings.alarm_type;
        gDBTClockController.targetDate = alarmSettings.targetDate;
        gDBTClockController.origTargetDate = alarmSettings.origTargetDate;
        gDBTClockController.countdown_time = alarmSettings.countdown_time;
        gDBTClockController.target_period = alarmSettings.target_period;
        gDBTClockController.period_offset = alarmSettings.period_offset;
        gDBTClockController.reminder_msg = alarmSettings.reminder_msg;
        gDBTClockController.snooze_increment = alarmSettings.snooze_increment;

        if (gDBTUserPreferenceInfo.getPrefValue("use_fixed_snooze"))
          gDBTClockController.snooze_increment = gDBTUserPreferenceInfo.getPrefValue("snooze_increment");

        document.getElementById("dbt-alarmicon-img").setAttribute("disabled", "false");
        document.getElementById("dbt-alarm-mitem").setAttribute("checked", "true");

        var alarmtime_str = "Alarm Time: " + dateToClock(gDBTClockController.targetDate);
        document.getElementById("dbt-alarmicon-img").setAttribute("tooltiptext", alarmtime_str);
      }
   }
  else
   {
     gDBTClockController.alarmState = gDBTconstants.AS_OFF;
     document.getElementById("dbt-alarmicon-img").setAttribute("disabled", "true");
     document.getElementById("dbt-alarmicon-img").removeAttribute("tooltiptext");
     document.getElementById("dbt-alarmicon-img").setAttribute("alarming", "false");
     document.getElementById("dbt-alarm-mitem").setAttribute("checked", "false");

     if (gAlarm_TimerID != null)
       clearInterval(gAlarm_TimerID);
   }
}


function toggleTimeDisplay()
{
  gDBTClockController.timeDisplayType = (gDBTClockController.timeDisplayType + 1) % 3;

  var currenttime = new Date();
  var clock_display = document.getElementById("dbt-menubarclock-lbl");

  switch (gDBTClockController.timeDisplayType)
  {
    case gDBTconstants.DT_TIME:
      clock_display.value = dateToClock(currenttime);
      clock_display.setAttribute("tooltiptext", "current time");
      break;
    case gDBTconstants.DT_DATE:
      clock_display.value = shortDate(currenttime);
      clock_display.setAttribute("tooltiptext", "current date");
      break;
    case gDBTconstants.DT_REMAIN:
      if (gDBTClockController.alarmState == gDBTconstants.AS_SET)
        clock_display.value = calculateTimeRemaining(currenttime);
      else
        clock_display.value = "00:00";
      clock_display.setAttribute("tooltiptext", "time remaining");
      break;
    default:
      break;
  }

}


/*****************************************************************
*  Snooze - Hitting snooze will bump back alarm time to (current *
*           time + snooze increment) if in ALARM state, or to    *
*           (alarm time + snooze increment) if in TIMING state.  *
*                                                                *
*****************************************************************/

function Snooze()
{
  if (gDBTClockController.alarmState != gDBTconstants.AS_OFF)
   {
     var alarmtime_str;
     var currenttime = new Date();

     if (gDBTClockController.alarmState == gDBTconstants.AS_ALARMING)
      {
        gDBTClockController.targetDate.setTime(currenttime.getTime() + gDBTClockController.snooze_increment * 60 * 1000);
        gDBTClockController.alarmState = gDBTconstants.AS_SET;
        document.getElementById("dbt-alarmicon-img").setAttribute("alarming", "false");

        if (gAlarm_TimerID != null)
          clearInterval(gAlarm_TimerID);
      }
     else // timerController.state == gDBTconstants.AS_SET
      {
        gDBTClockController.targetDate.setTime(gDBTClockController.targetDate.getTime() + gDBTClockController.snooze_increment * 60 * 1000);
      }

     alarmtime_str = "Alarm Time: " + dateToClock(gDBTClockController.targetDate);
     document.getElementById("dbt-alarmicon-img").setAttribute("tooltiptext", alarmtime_str);
   }
}


function dbtShowAlarmInfo()
{
  window.openDialog('chrome://derbrowsertimer/content/dialogAlarmInfo.xul',
                    'dbtAlarmInfo','chrome,modal,centerscreen');
}
