/****************************************************************************
**  DerBrowserTimer - Timer/Clock/Alarm add-on for Mozilla
**  Copyright (c) 2003  Charles Melhorn
**
**  This file is part of DerBrowserTimer.
**
**  DerBrowserTimer is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  DerBrowserTimer is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 

const gDBTPrefConstants =
{
  PT_BOOL:         0,  // preference types
  PT_INT:          1,
  PT_STR:          2,
  PT_UNICHAR:      3,
  PT_UNICHARLOCAL: 4,
  PT_FILE:         5
}

var gDBTPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);

const gDBTUserPreferenceInfo = 
{
  clock_format:      {defvalue: 0, prefname: "clockformat", preftype: gDBTPrefConstants.PT_INT},
  cuckoo_interval:   {defvalue: 0, prefname: "cuckoointerval", preftype: gDBTPrefConstants.PT_INT},
  cuckoo_playsound:  {defvalue: true, prefname: "cuckooplaysound", preftype: gDBTPrefConstants.PT_BOOL},
  cuckoo_sound_file: {defvalue: "cc_cuckoo.wav", prefname: "cuckoosoundfile", preftype: gDBTPrefConstants.PT_STR},
  auto_reset:        {defvalue: false, prefname: "autoreset", preftype: gDBTPrefConstants.PT_BOOL},
  alarm_timeout:     {defvalue: 2, prefname: "alarmtimeout", preftype: gDBTPrefConstants.PT_INT},
  alarm_popwindow:   {defvalue: false, prefname: "alarmpopwindow", preftype: gDBTPrefConstants.PT_BOOL},
  alarm_playsound:   {defvalue: true, prefname: "alarmplaysound", preftype: gDBTPrefConstants.PT_BOOL},
  alarm_sound_type:  {defvalue: "standard", prefname: "alarmsoundtype", preftype: gDBTPrefConstants.PT_STR},
  alarm_sound_file:  {defvalue: "alarm_tada.wav", prefname: "alarmsoundfile", preftype: gDBTPrefConstants.PT_STR},
  use_fixed_snooze:  {defvalue: false, prefname: "usefixedsnooze", preftype: gDBTPrefConstants.PT_BOOL},
  snooze_increment:  {defvalue: 5, prefname: "snoozeincrement", preftype: gDBTPrefConstants.PT_INT},

  prefNode:          gDBTPrefService.getBranch("extensions.dbt."),

  getPrefValue:      function(prefId_str)
  {
    return (getDBTUserPreference(this[prefId_str].prefname,
                                this[prefId_str].defvalue,
                                this[prefId_str].preftype,
                                this.prefNode));
  },

  getPrefDefValue:   function(prefId_str)
  {
    return this[prefId_str].defvalue;
  },

  setPrefValue:      function(prefId_str, value)
  {
    setDBTUserPreference(this[prefId_str].prefname,
                        value,
                        this[prefId_str].preftype,
                        this.prefNode);
  }

}


function getDBTUserPreference(prefName, defValue, prefType, prefNode)
{
  var e, rv;

  switch(prefType)
   {
     case gDBTPrefConstants.PT_BOOL:

       try {rv = prefNode.getBoolPref(prefName);}
        catch (e) { rv = defValue; }        

       break;

     case gDBTPrefConstants.PT_INT:
       
       try {rv = prefNode.getIntPref(prefName);}
        catch (e) { rv = defValue; }        

       break;

     case gDBTPrefConstants.PT_STR:

       try {rv = prefNode.getCharPref(prefName);}
        catch (e) { rv = defValue; }        

       break;

     case gDBTPrefConstants.PT_UNICHAR:
     case gDBTPrefConstants.PT_UNICHARLOCAL:
     case gDBTPrefConstants.PT_FILE:

     default:
       alert("Preference type not implemented"); // DEBUG
   }

  return rv;
}


function setDBTUserPreference(prefName, prefValue, prefType, prefNode)
{
  switch(prefType)
   {
     case gDBTPrefConstants.PT_BOOL:
       prefNode.setBoolPref(prefName, prefValue);
       break;

     case gDBTPrefConstants.PT_INT:
       prefNode.setIntPref(prefName, prefValue);
       break;

     case gDBTPrefConstants.PT_STR:
       prefNode.setCharPref(prefName, prefValue);
       break;

     case gDBTPrefConstants.PT_UNICHAR:
     case gDBTPrefConstants.PT_UNICHARLOCAL:
     case gDBTPrefConstants.PT_FILE:

     default:
       alert("Preference type not implemented"); // DEBUG
   }
}


