/****************************************************************************
**  DerBrowserTimer - Timer/Clock/Alarm add-on for Mozilla
**  Copyright (c) 2003  Charles Melhorn
**
**  This file is part of DerBrowserTimer.
**
**  DerBrowserTimer is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  DerBrowserTimer is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 


function ConfigureDialogWindow(sel_option)
{ 
//  var announce_prev_btn = document.getElementById('announce-preview-btn');
  var standard_prev_btn = document.getElementById('standard-preview-btn');
  var custom_sound_tbox = document.getElementById('custom-sound-tbox');
  var sound_browse_btn = document.getElementById('sound-browse-btn');
  var custom_prev_btn = document.getElementById('custom-preview-btn');

//  announce_prev_btn.disabled = ((sel_option == "announce") ? false : true);
  standard_prev_btn.disabled = ((sel_option == "standard") ? false : true);
  custom_sound_tbox.disabled = ((sel_option == "custom") ? false : true);
  sound_browse_btn.disabled = ((sel_option == "custom") ? false : true);

  custom_prev_btn.disabled = !((sel_option == "custom") && (custom_sound_tbox.value != ""));
}


function browseSoundFiles(sfile_str)
{
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
           .createInstance(nsIFilePicker);

  fp.init(window, "Select a File", nsIFilePicker.modeOpen);
  fp.appendFilter("Audio Files","*.wav");

  var res = fp.show();

  if (res == nsIFilePicker.returnOK)
   {
     var custom_snd_file=fp.file;
     document.getElementById('custom-sound-tbox').value = fp.fileURL.spec;
     document.getElementById('custom-preview-btn').disabled = false;
   }

}


function playSoundFile(sfile_str)
{
  // this code does works for chrome:// and http:// URLs,
  // but not for file:// URLs

  var sample = Components.classes["@mozilla.org/sound;1"].createInstance();
  sample = sample.QueryInterface(Components.interfaces.nsISound);
  const SND_NETWORK_STD_CID = "@mozilla.org/network/standard-url;1";
  const SND_I_URL           = "nsIURL";
  const SND_URL             = new Components.Constructor(SND_NETWORK_STD_CID, SND_I_URL);
  var url                   = new SND_URL();
  url.spec                  = "chrome://derbrowsertimer/content/" + sfile_str;
  sample.play(url);

}


function playSoundURL(sURL_str)
{
  var sample = Components.classes["@mozilla.org/sound;1"].createInstance();
  sample = sample.QueryInterface(Components.interfaces.nsISound);
  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Components.interfaces.nsIIOService);
  var url = ioService.newURI(sURL_str, null, null);
  sample.play(url);

}


function PreviewStandardSound()
{
  var filename;
  var selected_sound = (document.getElementById('stand-sounds-lbox')).selectedItem;

  // clicking on list box scroll bar will remove selection,
  // so need to check that something is selected before playing

  if (selected_sound)
   {
     filename = selected_sound.value;
     playSoundFile(filename);
   }

}


function PreviewCustomSound()
{
  var soundfileURL_str = document.getElementById('custom-sound-tbox').value;

  if (soundfileURL_str != "")
   {
     playSoundURL(soundfileURL_str);
   }

}


function PreviewCuckooSound()
{
  var filename = (document.getElementById('cuckoo-snds-rgroup')).selectedItem.value;

  playSoundFile(filename);

}


function initPrefDialog()
{
  // settings panel

  var clockformat_rgroup = document.getElementById("clock-format-rgroup"); 
  var clockformat = gDBTUserPreferenceInfo.getPrefValue("clock_format");
  var ele_arr = clockformat_rgroup.getElementsByAttribute("value", String(clockformat));
  clockformat_rgroup.selectedItem = ele_arr[0];

  var cuckoointerval_rgroup = document.getElementById("cuckoo-interval-rgroup"); 
  var cuckoointerval = gDBTUserPreferenceInfo.getPrefValue("cuckoo_interval");
  var ele_arr = cuckoointerval_rgroup.getElementsByAttribute("value", String(cuckoointerval));
  cuckoointerval_rgroup.selectedItem = ele_arr[0];

  var cuckoosound_cb = document.getElementById("cuckoo-sound-cbox"); 
  cuckoosound_cb.checked = gDBTUserPreferenceInfo.getPrefValue("cuckoo_playsound");

  var cuckoosound_rgroup = document.getElementById("cuckoo-snds-rgroup"); 
  var cuckoosound = gDBTUserPreferenceInfo.getPrefValue("cuckoo_sound_file");
  var ele_arr = cuckoosound_rgroup.getElementsByAttribute("value", cuckoosound);
  cuckoosound_rgroup.selectedItem = ele_arr[0];
  enableCuckooSoundsGroup(cuckoosound_cb.checked);

  var alarmplaysound_cb = document.getElementById("alarm-playsound-cbox"); 
  alarmplaysound_cb.checked = gDBTUserPreferenceInfo.getPrefValue("alarm_playsound");

  var alarmpopup_cb = document.getElementById("alarm-popup-cbox"); 
  alarmpopup_cb.checked = gDBTUserPreferenceInfo.getPrefValue("alarm_popwindow");

  var alarm_timeout_tbox = document.getElementById('alarm-timeout-tbox');
  alarm_timeout_tbox.value = gDBTUserPreferenceInfo.getPrefValue("alarm_timeout");

  var autoreset_cb = document.getElementById("alarm-autoreset-cbox"); 
  autoreset_cb.checked = gDBTUserPreferenceInfo.getPrefValue("auto_reset");

  var fixedsnooze_cb = document.getElementById("use-fixed-snooze-cbox"); 
  fixedsnooze_cb.checked = gDBTUserPreferenceInfo.getPrefValue("use_fixed_snooze");

  var snooze_increment_tbox = document.getElementById('snooze-increment-tbox');
  snooze_increment_tbox.value = gDBTUserPreferenceInfo.getPrefValue("snooze_increment");
  snooze_increment_tbox.disabled = !fixedsnooze_cb.checked;


  // alarm sounds panel

  var alarmsounds_rgroup = document.getElementById("alarm-sounds-rgroup"); 
  var alarmsoundtype = gDBTUserPreferenceInfo.getPrefValue("alarm_sound_type");
  var ele_arr = alarmsounds_rgroup.getElementsByAttribute("value", alarmsoundtype);
  alarmsounds_rgroup.selectedItem = ele_arr[0];

  var stand_sounds_lbox = document.getElementById('stand-sounds-lbox');

  if (alarmsoundtype == "standard")
   {
     stand_sounds_lbox.value = gDBTUserPreferenceInfo.getPrefValue("alarm_sound_file");
     stand_sounds_lbox.ensureElementIsVisible(stand_sounds_lbox.selectedItem);
   }
  else if (alarmsoundtype == "custom")
   {
     var custom_sound_tbox = document.getElementById("custom-sound-tbox");
     custom_sound_tbox.value = gDBTUserPreferenceInfo.getPrefValue("alarm_sound_file");
   }

  ConfigureDialogWindow(alarmsoundtype);

}


function EnableSnoozeBox(bEnable)
{
  document.getElementById('snooze-increment-tbox').disabled = (bEnable ? false : true);

}


function enableCuckooSoundsGroup(bEnable)
{
  document.getElementById('cuckoo-snds-rgroup').disabled = !bEnable;
  document.getElementById('cuckoo1-preview-btn').disabled = !bEnable;
}


function doOK()
{
  var snooze_increment_str, snooze_increment_Int;


  // settings panel

  var clockformat_rgroup = document.getElementById("clock-format-rgroup"); 
  var clockformat = parseInt(clockformat_rgroup.selectedItem.value);
  gDBTUserPreferenceInfo.setPrefValue("clock_format", clockformat);

  var cuckoointerval_rgroup = document.getElementById("cuckoo-interval-rgroup"); 
  var cuckoointerval = parseInt(cuckoointerval_rgroup.selectedItem.value);
  gDBTUserPreferenceInfo.setPrefValue("cuckoo_interval", cuckoointerval);

  var cuckoosound_cb = document.getElementById("cuckoo-sound-cbox"); 
  gDBTUserPreferenceInfo.setPrefValue("cuckoo_playsound", cuckoosound_cb.checked);

  var cuckoosound_rgroup = document.getElementById("cuckoo-snds-rgroup"); 
  var cuckoosoundfile = cuckoosound_rgroup.selectedItem.value;
  gDBTUserPreferenceInfo.setPrefValue("cuckoo_sound_file", cuckoosoundfile);

  var alarmplaysound_cb = document.getElementById("alarm-playsound-cbox"); 
  gDBTUserPreferenceInfo.setPrefValue("alarm_playsound", alarmplaysound_cb.checked);

  var alarmpopup_cb = document.getElementById("alarm-popup-cbox"); 
  gDBTUserPreferenceInfo.setPrefValue("alarm_popwindow", alarmpopup_cb.checked);

  var alarm_timeout_tbox = document.getElementById('alarm-timeout-tbox');
  gDBTUserPreferenceInfo.setPrefValue("alarm_timeout", parseInt(alarm_timeout_tbox.value));

  var autoreset_cb = document.getElementById("alarm-autoreset-cbox"); 
  gDBTUserPreferenceInfo.setPrefValue("auto_reset", autoreset_cb.checked);

  var fixedsnooze_cb = document.getElementById("use-fixed-snooze-cbox"); 
  gDBTUserPreferenceInfo.setPrefValue("use_fixed_snooze", fixedsnooze_cb.checked);

  var snooze_increment_tbox = document.getElementById('snooze-increment-tbox');

  if (fixedsnooze_cb.checked == true)
   {
    snooze_increment_str = snooze_increment_tbox.value;

    if (snooze_increment_str == "")
      snooze_increment_Int = 0;
    else
      snooze_increment_Int = parseInt(snooze_increment_str);

    if (snooze_increment_Int < 1)  // minimum snooze value is 1
      snooze_increment_Int = 1;

    gDBTUserPreferenceInfo.setPrefValue("snooze_increment", snooze_increment_Int);
   }


  // alarm sounds panel

  var custom_sound_tbox = document.getElementById('custom-sound-tbox');
  var alarmsounds_rgroup = document.getElementById("alarm-sounds-rgroup"); 
  var alarmsoundtype = alarmsounds_rgroup.selectedItem.value;

  if ( (alarmsoundtype == "custom") && (custom_sound_tbox.value == ""))
   {
     alert("Please supply custom sound file name");
     return false;
   }
  else
   {
     gDBTUserPreferenceInfo.setPrefValue("alarm_sound_type", alarmsoundtype);

     if (alarmsoundtype == "standard")
      {
        var selected_sound = (document.getElementById('stand-sounds-lbox')).selectedItem;

        if (selected_sound)
          gDBTUserPreferenceInfo.setPrefValue("alarm_sound_file", selected_sound.value);
      }
     else if (alarmsoundtype == "custom")
      {
        var alarmsoundfile = document.getElementById("custom-sound-tbox").value;
        gDBTUserPreferenceInfo.setPrefValue("alarm_sound_file", alarmsoundfile);
      }
   }

  return true
}

